import unittest
import whodidwhat.path_functions


class TestFileNameFunctions(unittest.TestCase):

    def test_split_all(self):
        parts = whodidwhat.path_functions.split_all('foo/bar/daa.cpp')
        self.assertEqual(['foo', 'bar', 'daa.cpp'], parts)

    def test_get_all_folder_levels(self):
        self.assertEqual(['first'], whodidwhat.path_functions.get_all_folder_levels('first/foo.cpp'))
        self.assertEqual(['first', 'first/second'], whodidwhat.path_functions.get_all_folder_levels('first/second/foo.cpp'))

    def test_get_blame_name(self):
        self.assertEqual('https.svne.com.testsuite.test.robot',
                         whodidwhat.path_functions.get_blame_name('https://svne.com/testsuite/test.robot'))

    def test_get_blame_name_really_long_path(self):
        blame_name = whodidwhat.path_functions.get_blame_name('https://svne1.access.huawei.com/isource/svnroot/\
robotlte/trunk/testsuite/MMM/user_plane/LTE2243/LTE2243_A_j/\
LTE1234_N_j_Ab_208_53_target_pcell_and_2source_Scell_deletion_at_timing_C_when_Exchange_of_PCell_and_Scell_Intra_eNB_Inter-LSP_HO_switch_the_other_scell_to _new.robot')
        self.assertEqual(255, len(blame_name))
        self.assertEqual('s.huawei.com.isource.svnroot.\
robotlte.trunk.testsuite.MMM.user_plane.LTE2243.LTE2243_A_j.\
LTE1234_N_j_Ab_208_53_target_pcell_and_2source_Scell_deletion_at_timing_C_when_Exchange_of_PCell_and_Scell_Intra_eNB_Inter-LSP_HO_switch_the_other_scell_to _new.robot', blame_name)

if __name__ == '__main__':
    unittest.main()



MAX_LINUX_FILENAME_LEN = 255


def split_all(path):
    return path.split('/')


def get_all_folder_levels(path):
    folders = split_all(path)[:-1]
    folder_levels = []
    for i in range(1, len(folders) + 1):
        folder_levels.append('/'.join(folders[:i]))
    return folder_levels


def get_blame_name(server_name):
    blame_name = server_name.replace('://', '.')
    blame_name = blame_name.replace('/', '.')
    return blame_name[-MAX_LINUX_FILENAME_LEN:]



import xml.etree.cElementTree as ET
import sys
import os
import subprocess
import argparse
import re
from datetime import datetime

import statistics
import path_functions


class SvnFilter(object):

    def __init__(self):
        self._statistics = statistics.Statistics()

    def parse_parameters_and_filter(self, argv=None):
        parameters = self.parse_parameters(argv)
        self._statistics.set_exclude_patterns(parameters.exclude)
        if parameters.input_xml:
            self._input_xmls = [SvnLogText(parameters.input_xml.read())]
        else:
            self._input_xmls = self._get_xml_logs(parameters)
        filtered_element_tree = self.filter_logs_by_users(self._input_xmls, parameters)
        if parameters.blame_folder or parameters.combine_blame:
            self.blame_active_files(parameters, filtered_element_tree)
        self.write_statistics(parameters)

    def write_statistics(self, parameters):
        if parameters.statistics_file:
            if os.path.splitext(parameters.statistics_file)[-1].lower() in ('.htm', '.html'):
                self._statistics.set_printer(statistics.HtmlPrinter(parameters.blame_folder))
            with open(parameters.statistics_file, 'w') as statistics_file:
                statistics_file.write(self._statistics.get_full())
        else:
            print(self._statistics.get_full())

    def blame_active_files(self, parameters, filtered_et):
        active_files = self.find_active_files(filtered_et)
        total_blamed_lines = ''
        if parameters.blame_folder and not os.path.isdir(parameters.blame_folder):
            os.mkdir(parameters.blame_folder)
        for filename in active_files:
            try:
                blame_log = subprocess.check_output(self._blame_command(filename, parameters))
            except subprocess.CalledProcessError:
                continue
            team_blame, blamed_lines = self.blame_only_given_users(blame_log, filename)
            total_blamed_lines += blamed_lines
            self._write_blamefile(team_blame, filename, parameters)
        self._write_combined_blame(total_blamed_lines, parameters)

    def _blame_command(self, server_name, parameters):
        blame_command = ['svn', 'blame']
        if parameters.revision is not None:
            blame_command.extend(['-r', parameters.revision])
        return blame_command + [server_name]

    def _write_blamefile(self, team_blame, server_name, parameters):
        if self._statistics.get_changed_lines_by_files()[server_name]:
            if parameters.blame_folder:
                with open(os.path.join(parameters.blame_folder,
                                       path_functions.get_blame_name(server_name)), 'w') as blamefile:
                    blamefile.write(team_blame)

    def _write_combined_blame(self, total_blamed_lines, parameters):
        if parameters.combine_blame:
            with open(parameters.combine_blame, 'w') as combined_blame_file:
                combined_blame_file.write(total_blamed_lines)

    def get_server_name(self, filename, svnlogtexts):
        for svnlogtext in svnlogtexts:
            if svnlogtext.repository and svnlogtext.repository.prefix in filename:
                filename = filename.replace(svnlogtext.repository.prefix, '')
                filename = filename.lstrip('/').lstrip('\\')
                return self._merge_common_parts(svnlogtext.repository.url, filename)
        raise Exception('Server name for filename {} not known'.format(filename))

    def _merge_common_parts(self, repository, filename):
        repository_in_parts = path_functions.split_all(repository)
        filename_in_parts = path_functions.split_all(filename)
        for repo_part in repository_in_parts:
            if repo_part in filename_in_parts:
                filename_in_parts.remove(repo_part)
        filename = '/'.join(filename_in_parts) if filename_in_parts else ''
        return '/'.join([repository.rstrip('/'), filename.lstrip('/')])

    def find_active_files(self, et):
        root = et.getroot()
        for logentry in root.findall('logentry'):
            author = logentry.find('author').text
            self._statistics.add_commit_count(author)
            for path in logentry.find('paths'):
                if path.attrib['kind'] == 'file':
                    server_name = self.get_server_name(path.text, self._input_xmls)
                    self._statistics.add_commit_count_of_file(server_name)
        return self._statistics.get_committed_files()

    def blame_only_given_users(self, blame_log, server_name):
        blame_file_for_team = ''
        lines_by_team_in_file = ''
        revision_and_user = re.compile(r'\s*\d+\s+\S+')
        for line in blame_log.splitlines(True):
            username = line.split()[1]
            if username in self._userlist:
                blame_file_for_team += line
                lines_by_team_in_file += line[revision_and_user.match(line).end():]
                self._statistics.add_changed_line(server_name, username)
            else:
                blame_file_for_team += self._remove_username(line, username)
        return blame_file_for_team, lines_by_team_in_file

    def _remove_username(self, line, username):
        return line.replace(username, ' ' * len(username), 1)

    def parse_parameters(self, argv):
        argv = argv if argv is not None else sys.argv
        p = argparse.ArgumentParser('Filter svn and git repositories based on list of users')
        p.add_argument('--input-xml', help='path to svn xml log input', type=file)
        p.add_argument('--users-file', help='file of usernames given line-by-line', type=argparse.FileType('r'))
        p.add_argument('--input-svn-repos', help='file of svn repository paths given line-by-line', type=file)
        p.add_argument('--output-xml', help='path for writing filtered xml')
        p.add_argument('--blame-folder', help='folder to store blames of top committed files')
        p.add_argument('-r', '--revision', help='revision info in similar format as svn log uses')
        p.add_argument('--statistics-file', help='file to store statistics on the run instead of printing on screen')
        p.add_argument('--exclude', help='file name pattern to exclude from statistics and blame', action='append')
        p.add_argument('--combine-blame', help='combine all blamed lines by team into one giant file')
        return p.parse_args(argv[1:])

    def _get_xml_logs(self, parameters):
        repositories = self._read_repository_urls(parameters.input_svn_repos)
        return [SvnLogText.from_server(repository, parameters.revision) for repository in repositories]

    def _read_repository_urls(self, fileobj):
        repos = []
        for line in fileobj:
            if line.strip() and not line.strip().startswith('#'):
                components = line.strip().split()
                if len(components) == 1:
                    repos.append(RepositoryUrl(components[0]))
                else:
                    repos.append(RepositoryUrl(components[0], components[1]))
        return repos

    def filter_logs_by_users(self, xml_log, parameters):
        self._userlist = self.read_userlist(parameters.users_file)
        filtered_et, _ = self.get_logs_by_users(xml_log)
        if parameters.output_xml is not None:
            filtered_et.write(parameters.output_xml, encoding='UTF-8', xml_declaration=True)
        return filtered_et

    def read_userlist(self, userlist_file):
        users = []
        for line in userlist_file:
            if line.strip() and not line.strip().startswith('#'):
                users.append(line.strip())
        return sorted(users)

    def get_logs_by_users(self, xml_logs):
        result_et, result_root = self._combine_logs_from_all_xmls_by_users(xml_logs, self._userlist)
        return self._sort_combined_tree_by_date(result_et, result_root)

    def _combine_logs_from_all_xmls_by_users(self, xml_logs, users):
        source_roots = [ET.fromstring(xml_log.log_text) for xml_log in xml_logs]
        result_root = ET.Element('log')
        result_et = ET.ElementTree(element=result_root)
        for root, xml_log in zip(source_roots, xml_logs):
            for logentry in root.findall('logentry'):
                if logentry.find('author').text in users:
                    self._prefix_paths_by_url_prefix(logentry, xml_log)
                    result_root.append(logentry)
        return result_et, result_root

    def _prefix_paths_by_url_prefix(self, logentry, xml_log):
        for path in logentry.find('paths'):
            if xml_log.repository:
                path.text = '/' + xml_log.repository.prefix + '/' + path.text[1:]

    def _sort_combined_tree_by_date(self, result_et, result_root):
        logentries = result_root.getchildren()

        def get_datetime(logentry):
            return datetime.strptime(logentry.find('date').text, '%Y-%m-%dT%H:%M:%S.%fZ')
        result_root[:] = sorted(logentries, key=get_datetime)
        return result_et, result_root


class RepositoryUrl(object):

    def __init__(self, url, prefix=''):
        self.url = url
        self.prefix = prefix


class SvnLogText(object):

    def __init__(self, log_text, repository=None):
        self.log_text = log_text
        self.repository = repository

    @classmethod
    def from_server(cls, repository, revision):
        svn_command = ['svn', 'log', '-v', '--xml']
        if revision:
            svn_command.extend(['-r', revision])
        svn_command.append('{}'.format(repository.url))
        return cls(subprocess.check_output(svn_command), repository)




from collections import defaultdict
import xml.etree.ElementTree as ET
import fnmatch
import os
import path_functions


class Statistics(object):

    def __init__(self):
        self._blamed_lines_by_file = defaultdict(lambda: 0)
        self._blamed_lines_by_user = defaultdict(lambda: 0)
        self._commit_counts_by_file = defaultdict(lambda: 0)
        self._commit_counts_by_user = defaultdict(lambda: 0)
        self.exclude_patterns = None
        self.printer = TextPrinter()

    def set_printer(self, printer):
        self.printer = printer

    def set_exclude_patterns(self, patterns):
        self.exclude_patterns = patterns

    def add_changed_line(self, server_name, author):
        if not self._is_excluded_file(server_name):
            self._blamed_lines_by_file[server_name] += 1
            self._blamed_lines_by_user[author] += 1

    def add_commit_count(self, author):
        self._commit_counts_by_user[author] += 1

    def add_commit_count_of_file(self, filename):
        if not self._is_excluded_file(filename):
            self._commit_counts_by_file[filename] += 1

    def _is_excluded_file(self, filename):
        if self.exclude_patterns is None:
            return False
        for pattern in self.exclude_patterns:
            if fnmatch.fnmatch(filename, pattern):
                return True
        return False

    def get_changed_lines_by_files(self):
        return self._blamed_lines_by_file

    def get_committed_files(self):
        return sorted(self._commit_counts_by_file, key=self._commit_counts_by_file.get, reverse=True)

    def get_commit_counts_by_files(self):
        return self._commit_counts_by_file

    def get_commit_counts_by_users(self):
        return self._commit_counts_by_user

    def get_changed_lines_by_users(self):
        return self._blamed_lines_by_user

    def get_changed_lines_by_folders(self):
        return self._get_changes_by_folders(self._blamed_lines_by_file)

    def _get_changes_by_folders(self, changes):
        folder_changes = []
        for f in changes:
            folders = path_functions.get_all_folder_levels(f)
            for level, folder in enumerate(folders):
                if len(folder_changes) <= level:
                    folder_changes.append(defaultdict(lambda: 0))
                folder_changes[level][folder] += changes[f]
        return folder_changes

    def get_commit_counts_by_folders(self):
        return self._get_changes_by_folders(self._commit_counts_by_file)

    def get_full(self):
        return self.printer.get_full(self)


class HtmlPrinter(object):

    def __init__(self, blame_folder):
        self._blame_folder = blame_folder

    def get_full(self, statistics):
        html = ET.Element('html')
        head = ET.SubElement(html, 'head')
        style = ET.SubElement(head, 'style')
        style.text = '''\
table {
    border-collapse: collapse;
}

table, td, th {
    border: 1px solid black;
}'''
        body = ET.SubElement(html, 'body')
        body.append(self.top_changed_lines_by_user_header())
        body.append(self.write(statistics.get_changed_lines_by_users()))
        body.append(self.top_commit_counts_by_user_header())
        body.append(self.write(statistics.get_commit_counts_by_users()))
        body.append(self.top_changed_lines_in_folders_header())
        body.append(self.write_folders(statistics.get_changed_lines_by_folders()))
        body.append(self.top_commit_counts_in_folders_header())
        body.append(self.write_folders(statistics.get_commit_counts_by_folders()))
        body.append(self.top_changed_lines_in_files_header())
        body.append(self.write_links(statistics.get_changed_lines_by_files()))
        body.append(self.top_commit_counts_in_files_header())
        body.append(self.write_links(statistics.get_commit_counts_by_files()))
        return ET.tostring(html, method='html')

    def write(self, statistic, limit=None):
        table = ET.Element('table')
        for item in sorted(statistic.items(), key=lambda it: (-it[1], it[0]))[:limit]:
            row = ET.SubElement(table, 'tr')
            row.append(self._create_element('td', text=str(item[0])))
            row.append(self._create_element('td', text=str(item[1])))
        return table

    def write_links(self, statistic, limit=None):
        table = ET.Element('table')
        for item in sorted(statistic.items(), key=lambda it: (-it[1], it[0]))[:limit]:
            row = ET.SubElement(table, 'tr')
            file_element = ET.SubElement(row, 'td')
            file_element.append(self._create_element('a', text=item[0], href=item[0]))
            if self._blame_folder is not None:
                blame_file = '/'.join([self._blame_folder, path_functions.get_blame_name(item[0])])
                if os.path.exists(blame_file):
                    blame_element = ET.SubElement(row, 'td')
                    blame_element.append(self._create_element('a', text='blame', href=blame_file))
            row.append(self._create_element('td', text=str(item[1])))
        return table

    def write_folders(self, level_list):
        table = ET.Element('table')
        for i, folder_level in enumerate(level_list):
            row = ET.SubElement(table, 'tr')
            row.append(self._create_element('td', text='level'))
            row.append(self._create_element('td', text=str(i + 1)))
            subtable = ET.SubElement(row, 'td')
            subtable.append(self.write_links(folder_level, 7))
        return table

    def top_changed_lines_by_user_header(self):
        return self._create_element('h2', text='Top changed lines by user')

    def top_commit_counts_by_user_header(self):
        return self._create_element('h2', text='Top commit counts by user')

    def top_changed_lines_in_folders_header(self):
        return self._create_element('h2', text='Top changed lines in folders')

    def top_commit_counts_in_folders_header(self):
        return self._create_element('h2', text='Top commit counts in folders')

    def top_changed_lines_in_files_header(self):
        return self._create_element('h2', text='Top changed lines in files')

    def top_commit_counts_in_files_header(self):
        return self._create_element('h2', text='Top commit counts in files')

    def _create_element(self, tag, text='', **attribs):
        e = ET.Element(tag)
        if text:
            e.text = text
        for attrib in attribs:
            e.set(attrib, attribs[attrib])
        return e


class TextPrinter(object):

    def get_full(self, statistics):
        full = self.top_changed_lines_by_user_header()
        full += self.write(statistics.get_changed_lines_by_users())
        full += self.top_commit_counts_by_user_header()
        full += self.write(statistics.get_commit_counts_by_users())
        full += self.top_changed_lines_in_folders_header()
        full += self.write_folders(statistics.get_changed_lines_by_folders())
        full += self.top_commit_counts_in_folders_header()
        full += self.write_folders(statistics.get_commit_counts_by_folders())
        full += self.top_changed_lines_in_files_header()
        full += self.write(statistics.get_changed_lines_by_files())
        full += self.top_commit_counts_in_files_header()
        full += self.write(statistics.get_commit_counts_by_files())
        return full

    def write(self, statistic, limit=None):
        text = ''
        limit = limit if limit is not None else len(statistic)
        for item in sorted(statistic.items(), key=lambda it: (-it[1], it[0]))[:limit]:
            text += '{}: {}\n'.format(item[0], item[1])
        return text

    def write_folders(self, level_list):
        total_text = ''
        for i, folder_level in enumerate(level_list):
            total_text += '---------------- level {} ---------------------------------\n'.format(i + 1)
            total_text += self.write(folder_level, 7)
        return total_text

    def top_changed_lines_by_user_header(self):
        return '''\
==========================================================
Top changed lines by user:
'''

    def top_commit_counts_by_user_header(self):
        return '''\
==========================================================
Top commit counts by user:
'''

    def top_changed_lines_in_folders_header(self):
        return '''\
==========================================================
Top changed lines in folders:
'''

    def top_commit_counts_in_folders_header(self):
        return '''\
==========================================================
Top committed folders:
'''

    def top_changed_lines_in_files_header(self):
        return '''\
==========================================================
Top changed lines in files:
'''

    def top_commit_counts_in_files_header(self):
        return '''\
==========================================================
Top commit counts in files:
'''



from whodidwhat.statistics import Statistics, HtmlPrinter
from whodidwhat.path_functions import get_blame_name
import unittest
import xml.etree.ElementTree as ET
import mock

class TestStatistics(unittest.TestCase):

    def setUp(self):
        self.statistics = Statistics()
        self.statistics.add_changed_line('trunk/file1', 'jkohvakk')
        self.statistics.add_changed_line('branch/file2', 'jkohvakk')
        self.statistics.add_changed_line('branch/file2', 'jkohvakk')
        self.statistics.add_commit_count_of_file('trunk/file1')
        self.statistics.add_commit_count_of_file('branch/file2')
        self.statistics.add_commit_count('jkohvakk')

        self.statistics.add_changed_line('branch/file2', 'kmikajar')
        self.statistics.add_commit_count('kmikajar')
        self.statistics.add_commit_count_of_file('branch/file2')

        self.statistics.add_changed_line('spike/deep_nesting/file2', 'jkohvakk')
        self.statistics.add_commit_count('jkohvakk')
        self.statistics.add_commit_count_of_file('spike/deep_nesting/file2')

    def test_changed_lines_by_files_text(self):
        self.assertEqual('''\
branch/file2: 3
spike/deep_nesting/file2: 1
trunk/file1: 1
''', self.statistics.printer.write(self.statistics.get_changed_lines_by_files()))

    def test_changed_lines_by_folders_text(self):
        self.assertEqual('''\
---------------- level 1 ---------------------------------
branch: 3
spike: 1
trunk: 1
---------------- level 2 ---------------------------------
spike/deep_nesting: 1
''', self.statistics.printer.write_folders(self.statistics.get_changed_lines_by_folders()))

    def test_changed_lines_by_users_text(self):
        self.assertEqual('''\
jkohvakk: 4
kmikajar: 1
''', self.statistics.printer.write(self.statistics.get_changed_lines_by_users()))

    def test_commit_counts_by_files_text(self):
        self.assertEqual('''\
branch/file2: 2
spike/deep_nesting/file2: 1
trunk/file1: 1
''', self.statistics.printer.write(self.statistics.get_commit_counts_by_files()))

    def test_commit_counts_by_users_text(self):
        self.assertEqual('''\
jkohvakk: 2
kmikajar: 1
''', self.statistics.printer.write(self.statistics.get_commit_counts_by_users()))

    def test_commit_counts_by_folders_text(self):
        self.assertEqual('''\
---------------- level 1 ---------------------------------
branch: 2
spike: 1
trunk: 1
---------------- level 2 ---------------------------------
spike/deep_nesting: 1
''', self.statistics.printer.write_folders(self.statistics.get_commit_counts_by_folders()))

    def test_get_full(self):
        self.assertEqual('''\
==========================================================
Top changed lines by user:
jkohvakk: 4
kmikajar: 1
==========================================================
Top commit counts by user:
jkohvakk: 2
kmikajar: 1
==========================================================
Top changed lines in folders:
---------------- level 1 ---------------------------------
branch: 3
spike: 1
trunk: 1
---------------- level 2 ---------------------------------
spike/deep_nesting: 1
==========================================================
Top committed folders:
---------------- level 1 ---------------------------------
branch: 2
spike: 1
trunk: 1
---------------- level 2 ---------------------------------
spike/deep_nesting: 1
==========================================================
Top changed lines in files:
branch/file2: 3
spike/deep_nesting/file2: 1
trunk/file1: 1
==========================================================
Top commit counts in files:
branch/file2: 2
spike/deep_nesting/file2: 1
trunk/file1: 1
''', self.statistics.get_full())

    def test_exclude_pattern(self):
        s = Statistics()
        s.set_exclude_patterns(['*.xml', '*.txt'])
        s.add_changed_line('file.xml', 'jkohvakk')
        s.add_changed_line('file.c', 'jkohvakk')
        s.add_changed_line('file.txt', 'kmikajar')
        s.add_commit_count('jkohvakk')
        s.add_commit_count('kmikajar')
        s.add_commit_count_of_file('file.xml')
        s.add_commit_count_of_file('file.c')
        s.add_commit_count_of_file('file.txt')
        self.assertEqual(1, s.get_commit_counts_by_files()['file.c'])
        self.assertEqual(0, s.get_commit_counts_by_files()['file.xml'])
        self.assertEqual(0, s.get_commit_counts_by_files()['file.txt'])
        self.assertEqual(1, s.get_changed_lines_by_files()['file.c'])
        self.assertEqual(0, s.get_changed_lines_by_files()['file.xml'])
        self.assertEqual(0, s.get_changed_lines_by_files()['file.txt'])
        self.assertEqual(1, s.get_changed_lines_by_users()['jkohvakk'])
        self.assertEqual(0, s.get_changed_lines_by_users()['kmikajar'])

    def test_limit_in_to_text(self):
        s = Statistics()
        data = {'foo': 4, 'bar': 10, 'daa': 5, 'dii': 2}
        self.assertEqual('''\
bar: 10
daa: 5
foo: 4
''', s.printer.write(data, 3))


class TestHtmlStatistics(unittest.TestCase):

    def setUp(self):
        self.statistics = Statistics()
        self.statistics.add_changed_line('trunk/file1', 'jkohvakk')
        self.statistics.add_changed_line('branch/file2', 'jkohvakk')
        self.statistics.add_changed_line('branch/file2', 'jkohvakk')
        self.statistics.add_commit_count_of_file('trunk/file1')
        self.statistics.add_commit_count_of_file('branch/file2')
        self.statistics.add_commit_count('jkohvakk')

        self.statistics.add_changed_line('branch/file2', 'kmikajar')
        self.statistics.add_commit_count('kmikajar')
        self.statistics.add_commit_count_of_file('branch/file2')

        self.statistics.add_changed_line('spike/deep_nesting/file2', 'jkohvakk')
        self.statistics.add_commit_count('jkohvakk')
        self.statistics.add_commit_count_of_file('spike/deep_nesting/file2')
        self.statistics.set_printer(HtmlPrinter(None))

    def test_changed_lines_by_users(self):
        self.assertEqual('''\
<table>\
<tr><td>jkohvakk</td><td>4</td></tr>\
<tr><td>kmikajar</td><td>1</td></tr>\
</table>''', ET.tostring(self.statistics.printer.write(self.statistics.get_changed_lines_by_users()),
                         method='html'))

    def test_changed_lines_by_files_no_blame(self):
        self.assertEqual('''\
<table>\
<tr><td><a href="branch/file2">branch/file2</a></td><td>3</td></tr>\
<tr><td><a href="spike/deep_nesting/file2">spike/deep_nesting/file2</a></td><td>1</td></tr>\
<tr><td><a href="trunk/file1">trunk/file1</a></td><td>1</td></tr>\
</table>''', ET.tostring(self.statistics.printer.write_links(self.statistics.get_changed_lines_by_files()),
                         method='html'))

    @mock.patch('whodidwhat.statistics.os.path.exists')
    def test_changed_lines_by_files_with_blames(self, os_path_exists):
        os_path_exists.return_value = True
        self.statistics.set_printer(HtmlPrinter('blame'))
        self.assertEqual('''\
<table>\
<tr><td><a href="branch/file2">branch/file2</a></td><td><a href="blame/branch.file2">blame</a></td><td>3</td></tr>\
<tr><td><a href="spike/deep_nesting/file2">spike/deep_nesting/file2</a></td><td><a href="blame/spike.deep_nesting.file2">blame</a></td><td>1</td></tr>\
<tr><td><a href="trunk/file1">trunk/file1</a></td><td><a href="blame/trunk.file1">blame</a></td><td>1</td></tr>\
</table>''', ET.tostring(self.statistics.printer.write_links(self.statistics.get_changed_lines_by_files()),
                         method='html'))

    def test_commit_counts_by_folders(self):
        self.assertEqual('''\
<table>\
<tr>\
<td>level</td><td>1</td>\
<td><table>\
<tr><td><a href="branch">branch</a></td><td>2</td></tr>\
<tr><td><a href="spike">spike</a></td><td>1</td></tr>\
<tr><td><a href="trunk">trunk</a></td><td>1</td></tr>\
</table></td>\
</tr>\
<tr>\
<td>level</td><td>2</td>\
<td><table>\
<tr><td><a href="spike/deep_nesting">spike/deep_nesting</a></td><td>1</td></tr>\
</table></td>\
</tr>\
</table>\
''', ET.tostring(self.statistics.printer.write_folders(self.statistics.get_commit_counts_by_folders()),
                 method='html'))

    def test_top_changed_lines_by_user_header(self):
        self.assertEqual('<h2>Top changed lines by user</h2>',
                         ET.tostring(self.statistics.printer.top_changed_lines_by_user_header()))

    def test_top_commit_counts_by_user_header(self):
        self.assertEqual('<h2>Top commit counts by user</h2>',
                         ET.tostring(self.statistics.printer.top_commit_counts_by_user_header()))

    def test_top_changed_lines_in_folders_header(self):
        self.assertEqual('<h2>Top changed lines in folders</h2>',
                         ET.tostring(self.statistics.printer.top_changed_lines_in_folders_header()))

    def test_top_commit_counts_in_folders_header(self):
        self.assertEqual('<h2>Top commit counts in folders</h2>',
                         ET.tostring(self.statistics.printer.top_commit_counts_in_folders_header()))

    def test_top_changed_lines_in_files_header(self):
        self.assertEqual('<h2>Top changed lines in files</h2>',
                         ET.tostring(self.statistics.printer.top_changed_lines_in_files_header()))

    def test_top_commit_counts_in_files_header(self):
        self.assertEqual('<h2>Top commit counts in files</h2>',
                         ET.tostring(self.statistics.printer.top_commit_counts_in_files_header()))

    @mock.patch('whodidwhat.statistics.os.path.exists')
    def test_write_full_html(self, os_path_exists):
        os_path_exists.return_value = True
        self.statistics.set_printer(HtmlPrinter('blame'))
        open('test.html', 'w').write(self.statistics.get_full())


if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()